using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;

namespace NeuroX
{

	/// <summary>
	/// claas representing one neuron
	/// </summary>
	public abstract class Neuron : ICloneable
	{
	
	
		/// <summary>
		/// Graphics path representing the nueron
		/// </summary>
		protected GraphicsPath gp;
		public bool bChoosen;
		/// <summary>
		/// Table of Points for drawing Neutrons
		/// </summary>
		protected Point[] points;


		/// <summary>
		/// // Point coordination
		/// </summary>
		public Point point;
		protected Pen myPen;
		protected Font myFont;
		protected SolidBrush drawBrush;
		public virtual void Paint(Graphics g)
		{
			g.DrawPath(myPen,gp);
			Font myFont= new Font("Arial", 8);
			if (bChoosen == true) g.DrawString("opt",myFont,drawBrush,new Point(point.X-20,point.Y));
			Double d = output;
			g.DrawString(d.ToString("0.##"),myFont,drawBrush,new Point(point.X,point.Y));

		}

		protected virtual void InitializeNeuron()
		{
			bChoosen = false;
			myPen = new Pen(Color.Red, 2);
			myFont= new Font("Arial", 8);
			drawBrush = new SolidBrush(Color.Green);
			gp = new GraphicsPath();
			points = new Point[3];
			points[0] = new Point(5,0);
			points[1] = new Point(-5,-10);
			points[2] = new Point(-5,10);
			gp = new GraphicsPath();			
			Matrix matrix =  new Matrix();
			matrix.Translate(point.X,point.Y);
			gp.AddPolygon(points);
			gp.Transform(matrix);
		}

		/// <summary>
		/// checks if Point is over Neuron
		/// </summary>
		public bool CheckIfOver(Point p)
		{
			if (gp.IsOutlineVisible(p,myPen)==true || gp.IsVisible(p) == true)
				return true;
			else return false;
		}

		/// <summary>
		/// // moves Neuron to Point p
		/// </summary>
		public void moveBy(Point p)
		{
			Matrix matrix =  new Matrix();
			matrix.Translate(p.X,p.Y);
			gp.Transform(matrix);
			point.X = p.X+point.X;
			point.Y = p.Y+point.Y;

		}

		public virtual double CalculateOutput()
		{
			output = 8;
			return 0;
		}
		protected double input;
		public int k;
		/// <summary>
		/// // set of neuron weights
		/// </summary>
		public Hashtable weights = new Hashtable();
		public double calculateInput(Hashtable neurons)
		{
			input = 0;
			IDictionaryEnumerator myEnumerator = weights.GetEnumerator();
			while ( myEnumerator.MoveNext() )
			{
				input+=((Neuron)neurons[myEnumerator.Key]).output*((Weight)myEnumerator.Value).dValue;
			}
			return input;
			
		}

		/// <summary>
		/// // output neuron value
		/// </summary>
		public double output;
		
		public abstract void randomParam(Random r);
		public abstract void reset(Random r);
		public abstract void reset();
		public abstract void standardParam();
		public void resetOutput(Random r)
		{
			output = -1 + r.NextDouble()*(2);
		}
		public void resetOutput()
		{
			output = 0;
		}
		
		public void initializeWeights(double min, double max, Random r)
		{
			ArrayList entryList = new ArrayList();
			foreach (DictionaryEntry myEntry in weights) 
			{
				entryList.Add(myEntry.Key);	
			}

			foreach (int i in entryList)
			{
				if (((Weight)weights[i]).bOptimize == true)
					((Weight)weights[i]).dValue =  min + r.NextDouble()*(max-min);
			}
//			output = min + r.NextDouble()*(max-min);
		}

		/// <summary>
		/// // if optimze this neuron
		/// </summary>
		public bool bOptimize;

		public abstract object Clone();

		protected object Clone(Neuron n)
		{
			n.weights = new Hashtable();
			IEnumerator keys = weights.Keys.GetEnumerator();
			while (keys.MoveNext())
			{
				n.weights.Add(keys.Current,((Weight)weights[keys.Current]).Clone());
			}
			n.input = input;
			n.output = output;
			n.bChoosen = bChoosen;
			n.bOptimize = bOptimize;
			n.k = k;
			return n;
		}
	}
}
